import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.List;


public class BouncerActor  extends Actor
{
    //MASK DATA
    private GreenfootImage mask;
    
    //WORLD DATA
    private double rotation=0;
    private double alpha = 0;
    private double elasticity = .5;
    
    //WANDERING DATA
    private int x,y;
    
    //BOOLEAN DATA
    private boolean recolor = false;
    private boolean active=true;
    private boolean selected=false;
    private boolean selectable = true;
    private boolean away = false;
    private static boolean hitHelp = false;
    
    //COLOR DATA
    private Color mainColor = Color.white;
    private static Color clear = new Color(0,0,0,0);
    
    //ADDEDTOWORLD****************************************************************************
    //  handles the masking and selectability
    
    protected void addedToWorld(World world){
        mask = new GreenfootImage(getImage().getWidth(),getImage().getHeight());
        for(int i=0;i<getImage().getWidth();i++){
            for(int j=0;j<getImage().getHeight();j++){
                if(getImage().getColorAt(i,j).equals(Color.white)){
                    getImage().setColorAt(i,j,clear);
                }else{
                    mask.setColorAt(i,j,Color.black);
                }
                if(getImage().getColorAt(i,j).equals(Color.red)){
                    getImage().setColorAt(i,j,clear);
                    mask.setColorAt(i,j,Color.red);
                }
            }
        }  
        if(isSelectable()){
            setAway();
        }
        if(recolor)
            recolor(mainColor);
        getImage().setTransparency(0);
        x = super.getX();
        y = super.getY();
    }
    
    //ACT****************************************************************************
    //  adds the objects mask to the worlds object buffer
    
    public void act() 
    {
        if(active && !isCollision()){
            alpha += (255-alpha)/20;
            ((worldness)getWorld()).drawObject(getX(),getY(),mask,getRotation());
        }else{
            if(!hitHelp && isCollision()){
                hitHelp = true;
                getWorld().addObject(new Helper(0),0,0);
            }
            alpha += (120-alpha)/7;
        }
        if(away){
            alpha += (70-alpha)/7;
        }
        getImage().setTransparency((int)alpha);
    }    
    
    //SETACTIVITY****************************************************************************
    //  sets whether or not the object is active or not
    
    public void setActivity(boolean a){
        active = a;
        away = false;
        if(a){
            getImage().setTransparency(255);
        }else{
            getImage().setTransparency(90);
        }
    }
    
    //GETREALROTATION****************************************************************************
    //  gets the more accurate double value for rotation
    
    public double getRealRotation(){
        return rotation;
    }
    
    //SETROTATION****************************************************************************
    //  sets the rotation of the object.  For full overiding of the rotation
    
    public void setRotation(int r){
        super.setRotation(r);
        rotation = r;
    }
    
    //SETROTATION****************************************************************************
    //  sets the rotation with a double value
    
    public void setRotation(double r){
        super.setRotation((int)r);
        rotation = r;
    }
    
    //ACTIVATE****************************************************************************
    //  is called when an object is hit.  returns whether or not the drop should continue to collide
    
    public boolean activate(drop b){
        return true;  
    }
    
    //SPECIAL****************************************************************************
    //  is called when the object is hit inside the special zone.  returns whether the drop should continue to collide
    
    public boolean special(drop b){
        return false;
    }
    
    //GETELASTICITY****************************************************************************
    //  returns the elasticity of this object
    
    public double getElasticity(drop b){
        return elasticity;
    }
    
    //SETELASTICITY****************************************************************************
    //  sets the elasticity of this object
    
    public void setElasticity(double e){
        elasticity = e;
    }
    
    //ISSELECTABLE****************************************************************************
    //  returns whether or not the object can be selected and moved/rotatede
    
    public boolean isSelectable(){
        return selectable;
    }
    
    //SETSELECTABLE****************************************************************************
    //  sets whether or not the object can be selected and moved/rotatede
    
    public void setSelectable(boolean s){
        selectable = s;
    }
    
    //ISCOLLISION****************************************************************************
    //  returns whether or not the object is currently colliding with another object
    
    public boolean isCollision(){
        return getOneIntersectingObject(BouncerActor.class)!=null && selectable;
    }
    
    //SETAWAY****************************************************************************
    //  sets the object as away and moves it back to the storage on the left
    
    public void setAway(){
        setLocation(20,80);
        while(getOneIntersectingObject(BouncerActor.class) != null){
            setLocation(20,getY()+50);
        }
        away = true;
        active = false;
        setRotation(0);
    }
    
    //RECOLOR****************************************************************************
    //  recolores the object with the worlds recoloring method
    
    public void recolor(Color c){
        setImage(worldness.recolor(getImage(),c));
    }
    
    //SETCOLOR****************************************************************************
    //  sets this object private color
    
    public void setColor(Color c){
        mainColor = c;
        recolor = true;
    }
    
    //SETLOCATION****************************************************************************
    //  overides the same method in ACTOR
    
    public void setLocation(int a,int b){
        x = a;
        y = b;
        super.setLocation(x,y);
    }
    
    //MOVE****************************************************************************
    //  provides a bit more movement functionality
    
    public void move(int a,int b){
        x += a;
        y += b;
        super.setLocation(x,y);
    }
    
    //GETX****************************************************************************
    //  returns x location
    
    public int getX(){
        return x;
    }
    
    //GETY****************************************************************************
    //  return Y location
    
    public int getY(){
        return y;
    }
    
    //ISAWAY****************************************************************************
    //  returns whether or not the object is currently active
    
    public boolean isActive(){
        return active;
    }
    
    //ISAWAY****************************************************************************
    //  returns whether or not the object is away
    
    public boolean isAway(){
        return away;
    }
    
    //GETMAINCOLOR****************************************************************************
    //  returns this objects main private color
    
    public Color getMainColor(){
        return mainColor;
    }
    
    //GETMASK****************************************************************************
    //  returns this objects object mask
    
    public GreenfootImage getMask(){
        return mask;
    }
    
    //FREEZE****************************************************************************
    //  freezes the object for transitions.  Is overided as needed
    
    public void freeze(){
    }
                    
}
