import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;


public class Text extends Actor
{
    //TEXT DATA
    private String[] text;
    
    //MANIPULATION DATA
    private int alpha=0;
    private int dir = 5;
    private int delta = 0;
    private int counter = 0;
    
    //BOOLEAN DATA
    private boolean finished = false;
    private boolean start = false;
    private boolean locked = false;
    
    //OBJECT DATA
    GreenfootImage over;
                      
    //TEXT****************************************************************************
    //  initializes this text display with an array of strings
    
    public Text(String[] t){
        text = t;
        getImage().clear();
    }
    
    //TEXT****************************************************************************
    //  initializes this text display with an array of strings
    
    public Text(){
        getImage().clear();
    }
    
    //ADDEDTOWORLD****************************************************************************
    //  gets the overlay object to draw onto
    
    protected void addedToWorld(World world){
        over = ((worldness)getWorld()).getOverlay().getImage();
    }
    
    //ACT****************************************************************************
    //  draws the text onto the overlay image.  Fades the image accordingly
    
    public void act() 
    {
        if(getWorld().getObjects(Text.class).size()<2){
            start = true;
        }
        
        if(!start){
            return;
        }

        over.setColor(new Color(255,255,255,alpha));
        for(int i=0;i<text.length;i++){
            over.drawString(text[i],getX()+delta+2,getY()+i*15);
            over.drawString(text[i],getX()+delta-1,getY()+i*15);
            over.drawString(text[i],getX()+delta,getY()+i*15+1);
            over.drawString(text[i],getX()+delta,getY()+i*15-1);
        }
        over.setColor(new Color(0,0,0,alpha));
        for(int i=0;i<text.length;i++){
            over.drawString(text[i],getX()+delta,getY()+i*15);
            over.drawString(text[i],getX()+delta+1,getY()+i*15);
        }
        alpha += dir;
        counter++;
        if(alpha==255){
            dir = 0;
        }
        if(Greenfoot.mousePressed(null) && counter>65 && !locked){
            dir = -5;
        }
        if(alpha==0){
            getWorld().removeObject(this);
            finished = true;
        }
    } 
    
    //ISFINISHED****************************************************************************
    //  returns whether or not the text is finished displaying
    
    public boolean isFinished(){
        return finished;
    }
    
    //MOVE****************************************************************************
    //  moves the position of the text box for transition compatability
    
    public void move(int x,int y){
        delta += x;
    }
    
    //SETSTRING****************************************************************************
    //  sets the display of this text object
    
    public void setString(String[] s){
        text = s;
    }
    
    //LOCK****************************************************************************
    //  locks the display from fading or going away
    
    public void lock(){
        locked = true;
    }
    
    //FADE****************************************************************************
    //  sets locked objects so that they fade out
    
    public void fade(){
        locked = false;
        dir = -5;
    }
}
