import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.List;

public class worldness  extends World
{
    //BUFFER DATA
    private GreenfootImage objectBuffer = new GreenfootImage(500,500);
    private GreenfootImage terrainBuffer = new GreenfootImage("TitleMask.gif");
    private GreenfootImage rotoBuffer = new GreenfootImage(100,100);
    private GreenfootImage overlayBuffer = new GreenfootImage(500,500);
    private overlay over = new overlay(overlayBuffer);
    
    private GreenfootImage dropImage = new GreenfootImage("drop.bmp");
    
    //SELECTION DATA
    private boolean selected = false;
    private boolean rotating = false;
    private boolean moving = false;
    private BouncerActor selectedActor;
    private int prevRot;
    private double prevClick = 0;
    
    //MOUSE DATA
    private int mx,my;
    private int offX,offY,offR;
    private MouseInfo mouse;
    private boolean pressed=false;
    private double pressTime=0;
    
    //MENU DATA
    private int counter=0;
    private boolean inMenu = true;
    private boolean fire = false;
    private Color[] fireColors = {Color.red,Color.yellow,Color.green,Color.magenta,Color.cyan};
    private int fireCounter = 0;
    private double time = System.currentTimeMillis();
    
    //GAME DATA
    private int toFill=1;
    private List cups = getObjects(Cup.class);
    private boolean terrainStatus = true;

    //WORLDNESS*****************************************************************************
    //  Initiates the wirld(to the main menu)
    
    public worldness()
    {    
        // Create a new world with 20x20 cells with a cell size of 10x10 pixels.
        super(500,500,1); 
        addObject(over,250,250);
        addObject(new Button(1),261,360);
        addObject(new Button(2),257,440);
        setBackground("TitleScreen.bmp");
        setActOrder(Switch.class,Pipe.class,plate.class,Hoop.class,spring.class,Cup.class,Door.class,cone.class,cannon.class,overlay.class,miniMenu.class,Button.class,drop.class,Display.class);
        setPaintOrder(miniMenu.class,Display.class,BouncerActor.class,overlay.class);
        
        for(int i=0;i<dropImage.getWidth();i++){
            for(int j=0;j<dropImage.getHeight();j++){
                if(dropImage.getColorAt(i,j).equals(Color.white)){
                    dropImage.setColorAt(i,j,new Color(0,0,0,0));
                }
            }
        }
    }
    
    //OK*****************************************************************************
    //  Used for collision detection
    
    
    public boolean ok(int a,int b){
        return !terrainStatus || (!objectBuffer.getColorAt(a,b).equals(Color.black) && !terrainBuffer.getColorAt(a,b).equals(Color.black));
    }
    
    public boolean okObject(int a,int b){
        return !terrainStatus || !objectBuffer.getColorAt(a,b).equals(Color.black);
    }
    
    public boolean objectSpecial(int a,int b){
        return !terrainStatus || objectBuffer.getColorAt(a,b).equals(Color.red);
    }
    
    //ACT****************************************************************************
    //  Main Act Method
    
    public void act(){
        counter++;
        checkNumber();
        
        if(inMenu){
            menu();
        }else{
            checkFill();
        }
        
        
        mouse = Greenfoot.getMouseInfo();
        if(mouse!=null){
            mx = mouse.getX();
            my = mouse.getY();
        }

        objectBuffer.setColor(Color.white);
        objectBuffer.fill();
        overlayBuffer.clear();
        overlayBuffer.setColor(Color.black);
        
        selection();
        
        
    }
    
    //CHECKNUMBER****************************************************************************
    //  makes sure the number of balls does not exceed 120
    
    public void checkNumber(){
        List drops = getObjects(drop.class);
        if(drops.size()>120){
            drop remove = null;
            int maxLife = 0;
            for(int i=0;i<drops.size();i++){
                if(((drop)drops.get(i)).getLife() > maxLife){
                    remove = (drop)drops.get(i);
                    maxLife = remove.getLife();
                }
            }
            if(remove!=null){
                removeObject(remove);     
            }
        }
    }
    
    //SELECTION****************************************************************************
    //  Handles selection of objects
    
    public void selection(){
        if(Greenfoot.mousePressed(null)){
            pressed = true;      
        }
        if(Greenfoot.mouseClicked(null)){
            if(System.currentTimeMillis()<pressTime && selected){
                selectedActor.setAway();
                selected = false;
            }
            pressTime = System.currentTimeMillis() + 300;
            if(moving || rotating){
                Greenfoot.playSound("Up.wav");
            }
            pressed = false;
            rotating = false;
            moving  = false;
            if(selected)
                selectedActor.setActivity(true);
        }  
        if(Greenfoot.mousePressed(null) && selected && !Greenfoot.mousePressed(selectedActor) && dist(mx,my,selectedActor.getX(),selectedActor.getY())<40){
            rotating = true;
            Greenfoot.playSound("Down.wav");
            offR = 40-(int)dist(mx,my,selectedActor.getX(),selectedActor.getY());
            prevRot = (int)(selectedActor.getRotation() - 180*Math.atan2(my-selectedActor.getY(),mx-selectedActor.getX())/Math.PI);
            prevClick = prevRot;
        }
        if(rotating){
            double mouseRot = 180*Math.atan2(my-selectedActor.getY(),mx-selectedActor.getX())/Math.PI;
            if(mouse!=null && Math.abs(prevClick - mouseRot)>3){
                Greenfoot.playSound("Roto.wav");
                prevClick = mouseRot;
            }
            overlayBuffer.setColor(Color.blue);
            
            selectedActor.setRotation(prevRot + mouseRot);
        }
        if(moving){
            overlayBuffer.setColor(Color.green);
            selectedActor.setLocation(mx+offX,my+offY);
            if(selectedActor.getY()<50){
                selectedActor.setLocation(selectedActor.getX(),50);
            }
            if(selectedActor.getY()>499){
                selectedActor.setLocation(selectedActor.getX(),499);
            }
            if(selectedActor.getX()<0){
                selectedActor.setLocation(0,selectedActor.getY());
            }
            if(selectedActor.getX()>499){
                selectedActor.setLocation(499,selectedActor.getY());
            }
        }     
        List bouncers = getObjects(BouncerActor.class);
        if(Greenfoot.mousePressed(null)){
            selected = rotating;
            for(int i=0;i<bouncers.size();i++){
                if(Greenfoot.mousePressed((BouncerActor)bouncers.get(i)) && ((BouncerActor)bouncers.get(i)).isSelectable()){
                    selectedActor = (BouncerActor)bouncers.get(i);
                    selected = true;
                    rotating = false;
                    moving = true;
                    offX = selectedActor.getX()-mx;
                    offY = selectedActor.getY()-my;
                    selectedActor.setActivity(false);
                }
            }
            if(moving){
                Greenfoot.playSound("Down.wav");
            }
        }
        if(selected){
            if(selectedActor.isCollision()){
                overlayBuffer.setColor(Color.red);
            }
            int r=80;
            if(rotating){
                r = 2*offR + 2*(int)dist(mx,my,selectedActor.getX(),selectedActor.getY());
            }
            overlayBuffer.drawOval(selectedActor.getX()-r/2,selectedActor.getY()-r/2,r,r);   
        }
    }
      
    //DRAWOBJECT****************************************************************************
    //  Draw objects to the object buffer
    
    public void drawObject(int x,int y,GreenfootImage image,double r){
        rotoBuffer.clear();
        rotoBuffer.drawImage(image,50-image.getWidth()/2,50-image.getHeight()/2);
        rotoBuffer.rotate((int)r);
        objectBuffer.drawImage(rotoBuffer,x-50,y-50);
    }
    
    //DIST****************************************************************************
    //  A simple helper class :)
    
    public double dist(double a,double b,double c,double d){
        return Math.sqrt(Math.pow(a-c,2)+Math.pow(b-d,2));
    }
    
    //MENU****************************************************************************
    //  menu code
    
    public void menu(){
        if(fire && counter%40==0){
            fireCounter++;
            if(counter%80 == 0){
                addObject(new firework(2*Math.random(),-6-Math.random(),fireColors[fireCounter%5]),20,480);
            }else{
                addObject(new firework(-2*Math.random(),-6-Math.random(),fireColors[fireCounter%5]),480,480);
            }
        }
        
        
        if(System.currentTimeMillis()-time > 1000 && !fire){
            Greenfoot.playSound("Rain.wav");
            time = System.currentTimeMillis();
        }
        if((counter%2==0 && !fire)){
            addObject(new drop(dropImage,Color.blue),Greenfoot.getRandomNumber(500),10);
            //addObject(new ball(),250,50);
        }
    }
    
    //QUITMENU****************************************************************************
    //  quit the menu and load the level code
    
    public void quitMenu(){
        terrainBuffer.clear();
        over.nextLevel();
        over.start();
    }
    
    //RECOLOR****************************************************************************
    //  static helper method
    
    public static GreenfootImage recolor(GreenfootImage image, Color c){
        Color temp;
        double mag;
        int red,green,blue;
        for(int i=0;i<image.getWidth();i++){
            for(int j=0;j<image.getHeight();j++){
                temp = image.getColorAt(i,j);
                if(!temp.equals(new Color(0,0,0,0))){
                    mag = Math.pow((temp.getRed() + temp.getGreen() + temp.getBlue())/765.0,2);
                    if(mag<.5){
                        red = (int)(2*mag*c.getRed());
                        green = (int)(2*mag*c.getGreen());
                        blue = (int)(2*mag*c.getBlue());
                    }else{
                        red = c.getRed();
                        green = c.getGreen();
                        blue = c.getBlue();
                        red += (int)((255-red)*(mag-.5)*2);
                        green += (int)((255-green)*(mag-.5)*2);
                        blue += (int)((255-blue)*(mag-.5)*2);
                        //blue = 255;
                    }
                    temp = new Color(red,green,blue);
                    image.setColorAt(i,j,temp);
                }
            }
        }
        return image;
    }
    
    //KILL(ALL)****************************************************************************
    //  two methods used for removing objects
    
    
    public void killAll(){
        inMenu = false;
        List things = getObjects(BouncerActor.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(drop.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(Button.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(Display.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(miniMenu.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(codes.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
    }
    
    public void kill(){
        inMenu = false;
        List things = getObjects(BouncerActor.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(drop.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(Text.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(Helper.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
        things = getObjects(codes.class);
        for(int i=0;i<things.size();i++){
            removeObject((Actor)things.get(i));
        }
    }
    
    //FILLERS****************************************************************************
    //  Keeps track of how many cups have been filled
    
    public void setFill(){
        cups = getObjects(Cup.class);
        toFill = cups.size();
    }
    
    public void checkFill(){
        int count=0;
        for(int i=0;i<cups.size();i++){
            count += ((Cup)cups.get(i)).isFull();
        }

        if(count==toFill){
            over.nextLevel();
            over.start();
            toFill = 0;
            selected = false;
        }
    }
    
    //RELOADMENU****************************************************************************
    //  Loads the menu from any given point
    
    public void reloadMenu(){
        over.setNextLevel(0);
        over.start();
        over.setKill(true);
        toFill = 23;
    }
    
    //TERRAINSTATUS****************************************************************************
    //  Keeps the balls from acting weird between levels
    
    public void setTerrainOff(){
        terrainStatus = false;
    }
    
    public void setTerrainOn(){
        terrainStatus = true;
    }
    
    public overlay getOverlay(){
        return over;
    }
    
    public void setSelection(boolean s){
        selected = s;
        moving = s;
        rotating = s;
    }
    
    public void setInMenu(boolean m){
        inMenu = m;
        if(inMenu){
            toFill = 1;
        }
    }
    
    public GreenfootImage getTerrainBuffer(){
        return terrainBuffer;
    }
    
    public void setTerrainBuffer(GreenfootImage t){
        terrainBuffer = t;
    }
    
    public void loadCode(String text){
        over.loadCode(text);
    }
    
    public void fireworks(){
        fire = true;
    }
    
}
